package com.uas.platform.warehouse.backend.config.interceptor;

import com.uas.platform.warehouse.backend.transport.auth.entity.AppAuthentication;
import com.uas.platform.warehouse.backend.transport.auth.service.AppAuthenticationService;
import com.uas.platform.warehouse.backend.transport.util.AuthenticationUtil;
import com.uas.platform.warehouse.backend.util.SignUtil;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.Validate;
import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

/**
 * Interceptor that validates request's availability.
 *
 * @author huxz
 */
public class AuthenticateInterceptor implements HandlerInterceptor {

  private static final Logger logger = Logger.getLogger(
      AuthenticateInterceptor.class);

  /**
   * Determine whether request time is failure.
   */
  private static final int FAILURE_INTERVAL = 5;

  private final AppAuthenticationService appAuthenticationService;

  public AuthenticateInterceptor(
      AppAuthenticationService appAuthenticationService) {
    this.appAuthenticationService = appAuthenticationService;
  }

  @Override
  public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
      throws Exception {
    if (logger.isDebugEnabled()) {
      logger.debug("Authenticate interceptor is triggered");
    }

    final String appKey = request.getParameter("appKey");
    final String timestamp = request.getParameter("timestamp");
    final String sign = request.getParameter("sign");

    Validate.notBlank(appKey, "App Key不能为空");
    Validate.notBlank(timestamp, "时间戳不能为空");
    Validate.notBlank(sign, "签名字符串不能为空");

    // 验证AppKey是否存在，并查询SecretKey
    String secretKey;
    AppAuthentication appAuthentication = appAuthenticationService
        .loadAuthenticationByAppKey(appKey);
    if (appAuthentication == null || StringUtils.isEmpty(appAuthentication.getAppKey())
        || !appAuthentication.isEnabled()) {
      throw new RuntimeException("App Key无效");
    } else {
      if (logger.isDebugEnabled()) {
        logger.debug(appAuthentication);
      }
      secretKey = appAuthentication.getSecretKey();
      if (secretKey == null || secretKey.length() == 0) {
        throw new RuntimeException("Secret Key无效");
      }
    }

    // 验证时间戳的有效性，避免同一个请求多次使用
    DateTime systemTime = new DateTime();
    DateTime requestTime = new DateTime(Long.valueOf(timestamp).longValue());
    if (logger.isDebugEnabled()) {
      String pattern = "yyyy-MM-dd HH:mm:ss.SSSZ";
      logger.debug(String.format("SystemTime: %s\n\tRequestTime: %s", systemTime.toString(pattern),
          requestTime.toString(pattern)));
    }

    if (systemTime.isAfter(requestTime.plusMinutes(FAILURE_INTERVAL))) {
      throw new RuntimeException("无效时间戳");
    }

    // 验证签名是否正确
/*    Map<String, String> params = new HashMap<>(16);
    Enumeration<String> parameterNames = request.getParameterNames();
    while (parameterNames.hasMoreElements()) {
      String key = parameterNames.nextElement();
      if (!"appKey".equals(key) && !"sign".equals(key)) {
        params.put(key, request.getParameter(key));
      }
    }*/
    String validateSign = SignUtil.createMD5Sign(appKey, secretKey, timestamp);
    System.out.println(validateSign);
    if (!sign.equals(validateSign)) {
      throw new RuntimeException("签名不一致");
    }

    // Save appAuthentication to LocalThread, avoid user to use other enterprise' uu
    AuthenticationUtil.setAuthentication(appAuthentication);

    return true;
  }

  @Override
  public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
      ModelAndView modelAndView) throws Exception {
    // Clear app authentication info
    AuthenticationUtil.removeAuthentication();
  }

  @Override
  public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
      Object handler, Exception ex) throws Exception {

  }
}
