package com.uas.platform.warehouse.backend.service.impl;

import com.uas.platform.warehouse.backend.dao.ErrorLogDao;
import com.uas.platform.warehouse.backend.entity.*;
import com.uas.platform.warehouse.backend.service.CommonService;
import com.uas.platform.warehouse.backend.service.ShipmentService;
import com.uas.platform.warehouse.backend.transport.auth.entity.AppAuthentication;
import com.uas.platform.warehouse.backend.transport.service.NotificationService;
import com.uas.platform.warehouse.backend.transport.service.PullDataService;
import com.uas.platform.warehouse.backend.transport.util.AuthenticationUtil;
import com.uas.platform.warehouse.core.util.JacksonUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Created by luhg on 2017/11/8.
 * 处理单据信息及回传信息.
 */
@Service
public class CommonServiceImpl implements CommonService{
    private final ErrorLogDao logDao;

    private final NotificationService notificationService;

    private final ShipmentService shipmentService;

    private final PullDataService pullDataService;

    @Autowired
    public CommonServiceImpl(
            ErrorLogDao logDao, ShipmentService shipmentService,
            PullDataService pullDataService,
            NotificationService notificationService) {
        this.logDao = logDao;
        this.shipmentService = shipmentService;
        this.pullDataService = pullDataService;
        this.notificationService = notificationService;
    }

    /**
     * 物料资料处理.
     * @param data
     * @return
     */
    public boolean doProduct(String data){
        boolean status = false;
        Product product = JacksonUtils.fromJson(data,Product.class);
        try {
            status = shipmentService.productFromEnterpriseForApi(product);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_VOUCHER, product.getReceiverUU(), data, e.getMessage());
        }
        return status;
    }

    /**
     * 保存错误信息.
     * @param type
     * @param receiverUu
     * @param data
     * @param message
     */
    public void saveErrorLogInfo(ErrorLog.LogType type, String receiverUu, String data, String message) {
        ErrorLog log = new ErrorLog();
        log.setCreateTime(DateTime.now().toDate());
        log.setType(type);
        AppAuthentication authentication = AuthenticationUtil.getAuthentication();
        log.setSenderUu(authentication.getEnUu());
        log.setReceiverUu(receiverUu);
        log.setData(data);
        log.setReason(message);

        System.out.println("data" + data);
        System.out.println("message" + message);

        logDao.save(log);
    }

    /**
     * 处理调拨单.
     * @param data
     * @return
     */
    public boolean doTransfer(String data){
        boolean status = false;
        Transfer transfer = JacksonUtils.fromJson(data,Transfer.class);
        try {
            status = shipmentService.transferFromWarehouseForApi(transfer);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_VOUCHER, transfer.getReceiverUU(), data, e.getMessage());
        }

        return status;
    }

    /**
     * 处理仓储服务商回复信息.
     * @param data
     * @return
     */
    public boolean doNormalReply(String data){
        NormalReply reply = JacksonUtils.fromJson(data, NormalReply.class);
        try {
            return shipmentService.replyFromWarehouse(reply);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_REPLY, reply.getReceiverUU(), data, e.getMessage());
        }
        return false;
    }

    /**
     * 处理出货单.
     * @param data
     * @return
     */
    public boolean doShipment(String data){
        boolean status = false;
        Shipment shipment = JacksonUtils.fromJson(data,Shipment.class);
        try {
            status = shipmentService.shipFromWarehouseForApi(shipment);
        } catch (RuntimeException e) {
            e.printStackTrace();
            saveErrorLogInfo(ErrorLog.LogType.INVALID_VOUCHER, shipment.getReceiverUU(), data, e.getMessage());
        }
        return status;
    }

    /**
     * 处理入库单.
     * @param data
     * @return
     */
    public boolean doStockin(String data){
        boolean status = false;
        Stockin stockin = JacksonUtils.fromJson(data,Stockin.class);
        try {
            status = shipmentService.stockinFromWarehouseForApi(stockin);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_VOUCHER, stockin.getReceiverUU(), data, e.getMessage());
        }
        return status;
    }

    /**
     * 处理包装单.
     * @param data
     * @return
     */
    public boolean doPacking(String data){
        boolean status = false;
        Packing packing = JacksonUtils.fromJson(data,Packing.class);
        try {
            status = shipmentService.packingFromWarehouseForApi(packing);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_VOUCHER, packing.getReceiverUU(), data, e.getMessage());
        }
        return status;
    }

    /**
     * 处理发票.
     * @param data
     * @return
     */
    public boolean doInvoice(String data){
        boolean status = false;
        Invoice invoice = JacksonUtils.fromJson(data,Invoice.class);
        try {
            status = shipmentService.invoiceFromWarehouseForApi(invoice);
        } catch (RuntimeException e) {
            e.printStackTrace();
            saveErrorLogInfo(ErrorLog.LogType.INVALID_VOUCHER, invoice.getReceiverUU(), data, e.getMessage());
        }
        return status;
    }

    /**
     * 处理包装单回复.
     * @param data
     * @return
     */
    public boolean doPackingReply(String data){
        PackingReply packingReply = JacksonUtils.fromJson(data,PackingReply.class);
        try {
            return shipmentService.replyPackingFromWarehouse(packingReply);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_REPLY, packingReply.getReceiverUU(), data, e.getMessage());
        }
        return false;
    }

    /**
     * 处理发票回复.
     * @param data
     * @return
     */
    public boolean doInvoiceReply(String data) {
        InvoiceReply invoiceReply = JacksonUtils.fromJson(data, InvoiceReply.class);
        try {
            return shipmentService.replyInvoiceFromWarehouse(invoiceReply);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_REPLY, invoiceReply.getReceiverUU(), data, e.getMessage());
        }
        return false;
    }

        /**
         * 处理出入库报关回复.
         * @param data
         * @return
         */
    public boolean doProdioMainfestReply(String data){
        ProdioMainfest mainfest = JacksonUtils.fromJson(data,ProdioMainfest.class);
        try {
            return shipmentService.replyProdioMainfestFromWarehouse(mainfest);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_REPLY, mainfest.getReceiverUU(), data, e.getMessage());
        }
        return false;
    }

    /**
     * 处理物料报关回复.
     * @param data
     * @return
     */
    public boolean doProductMainfestReply(String data){
        ProductMainfest mainfest = JacksonUtils.fromJson(data,ProductMainfest.class);
        try {
            return shipmentService.replyProductMainfestFromWarehouse(mainfest);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_REPLY, mainfest.getReceiverUU(), data, e.getMessage());
        }
        return false;
    }

    /**
     * 处理费用回复.
     * @param data
     * @return
     */
    public boolean doFeeReply(String data){
        Fee fee = JacksonUtils.fromJson(data,Fee.class);
        try {
            return shipmentService.replyFeeFromWarehouse(fee);
        } catch (RuntimeException e) {
            saveErrorLogInfo(ErrorLog.LogType.INVALID_REPLY, fee.getReceiverUU(), data, e.getMessage());
        }
        return false;
    }
}
