package com.uas.platform.warehouse.backend.service.impl;

import com.uas.platform.warehouse.backend.dao.*;
import com.uas.platform.warehouse.backend.entity.*;
import com.uas.platform.warehouse.backend.service.ShipmentService;
import com.uas.platform.warehouse.backend.transport.auth.entity.AppAuthentication;
import com.uas.platform.warehouse.backend.transport.auth.entity.AppAuthentication.IdentificationType;
import com.uas.platform.warehouse.backend.transport.entity.BaseVoucher;
import com.uas.platform.warehouse.backend.transport.entity.Notification;
import com.uas.platform.warehouse.backend.transport.entity.TransferStatus;
import com.uas.platform.warehouse.backend.transport.service.NotificationService;
import com.uas.platform.warehouse.backend.transport.util.AuthenticationUtil;
import com.uas.platform.warehouse.core.domain.VoucherType;
import org.apache.commons.lang3.Validate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Java Bean implementation of {@code StockInService}.
 *
 * @author huxz
 */
@Service
public class ShipmentServiceImpl implements ShipmentService {
  private final StockinDao stockinDao;
  private final ShipmentDao shipmentDao;
  private final TransferDao transferDao;
  private final ProductDao productDao;
  private final PackingDao packingDao;
  private final InvoiceDao invoiceDao;
  private final NormalReplyDao normalReplyDao;
  private final PackingReplyDao packingReplyDao;
  private final InvoiceReplyDao invoiceReplyDao;
  private final NotificationService notificationService;
  private final ProdioMainfestDao prodioMainfestDao;
  private final ProductMainfestDao productMainfestDao;
  private final FeeDao feeDao;

  @Autowired
  public ShipmentServiceImpl(ShipmentDao shipmentDao,
      StockinDao stockinDao,
      TransferDao transferDao,
      ProductDao productDao,
      PackingDao packingDao,
      NormalReplyDao normalReplyDao,
      PackingReplyDao packingReplyDao,
      InvoiceDao invoiceDao,
      InvoiceReplyDao invoiceReplyDao,
      ProdioMainfestDao prodioMainfestDao,
      ProductMainfestDao productMainfestDao,
      FeeDao feeDao,
      NotificationService notificationService) {
    this.stockinDao = stockinDao;
    this.shipmentDao = shipmentDao;
    this.transferDao = transferDao;
    this.productDao = productDao;
    this.packingDao = packingDao;
    this.invoiceDao = invoiceDao;
    this.normalReplyDao = normalReplyDao;
    this.packingReplyDao = packingReplyDao;
    this.invoiceReplyDao = invoiceReplyDao;
    this.prodioMainfestDao = prodioMainfestDao;
    this.productMainfestDao = productMainfestDao;
    this.notificationService = notificationService;
    this.feeDao = feeDao;
  }

  @Override
  @Transactional(rollbackFor = RuntimeException.class)
  public boolean shipFromWarehouseForApi(Shipment shipment) {
    //Validate.notBlank(whName, "仓库名称不能为空");
    Validate.notNull(shipment, "出货单不能为空");

    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    checkAppKey(authentication);

    shipment.setEnUu(authentication.getEnUu());

    shipmentDao.save(shipment);

    return saveDataToMongo(shipment);
  }

  @Override
  public boolean stockinFromWarehouseForApi(Stockin stockin) {
    Validate.notNull(stockin, "入库单不能为空");

    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    checkAppKey(authentication);

    stockin.setEnUu(authentication.getEnUu());

    stockinDao.save(stockin);

    return saveDataToMongo(stockin);
  }

  @Override
  public boolean transferFromWarehouseForApi(Transfer transfer) {
    //Validate.notBlank(whName, "仓库名称不能为空");
    Validate.notNull(transfer, "调拨单不能为空");

    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    checkAppKey(authentication);

    transfer.setEnUu(authentication.getEnUu());

    transferDao.save(transfer);

    return saveDataToMongo(transfer);
  }

  @Override
  public boolean productFromEnterpriseForApi(Product product) {
    //Validate.notBlank(whName, "仓库名称不能为空");
    Validate.notNull(product, "物料资料不能为空");

    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    checkAppKey(authentication);

    product.setEnUu(authentication.getEnUu());

    productDao.save(product);

    return saveDataToMongo(product);
  }

  /**
   * Save data of voucher to mongo when enterprise customer upload data.
   *
   * @param voucher voucher info
   * @return status of transferring voucher
   */
  private boolean saveDataToMongo(BaseVoucher voucher) {
    Validate.notNull(voucher.getType(), "单据类型不能为空");

    Notification notification = new Notification();
    // Set enterprise info
    notification.setEnUu(voucher.getEnUu());
    // Set warehouse provider info
    notification.setWhEnUu(voucher.getReceiverUU());
    //notification.setWhName(voucher.getWhName());
    notification.setVoucher(voucher);
    notification.setType(voucher.getType());

    notificationService.saveVoucherDataToMongo(notification);
    //return notification.getStatus();
    return true;
  }

  @Override
  @Transactional(rollbackFor = RuntimeException.class)
  public boolean replyFromWarehouse(NormalReply reply) {
    Validate.notNull(reply, "回传信息不能为空");

    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    if (authentication == null || IdentificationType.WAREHOUSE_PROVIDER != authentication.getType()) {
      throw new IllegalStateException("App Key无效");
    }

    reply.setSenderUU(authentication.getEnUu());

    normalReplyDao.save(reply);

    notificationService.replyFromWarehouse(reply);
    return true;
  }

  @Override
  public boolean packingFromWarehouseForApi(Packing packing) {
    //Validate.notBlank(whName, "仓库名称不能为空");
    Validate.notNull(packing, "包装单不能为空");

    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    checkAppKey(authentication);

    packing.setEnUu(authentication.getEnUu());

    packingDao.save(packing);

    return saveDataToMongo(packing);
  }

  @Override
  public boolean invoiceFromWarehouseForApi(Invoice invoice) {
    Validate.notNull(invoice, "发票不能为空");

    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    checkAppKey(authentication);

    invoice.setEnUu(authentication.getEnUu());

    invoiceDao.save(invoice);

    return saveDataToMongo(invoice);
  }

  @Override
  public boolean replyPackingFromWarehouse(PackingReply packingReply) {
    Validate.notNull(packingReply, "回传信息不能为空");
    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    if (authentication == null) {
      throw new IllegalStateException("App Key无效");
    }

    packingReply.setSenderUU(authentication.getEnUu());

    packingReplyDao.save(packingReply);

    notificationService.replyFromWarehouse(packingReply);
    return true;
  }

  @Override
  public boolean replyInvoiceFromWarehouse(InvoiceReply invoiceReply) {
    Validate.notNull(invoiceReply, "回传信息不能为空");
    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    if (authentication == null) {
      throw new IllegalStateException("App Key无效");
    }
    invoiceReply.setSenderUU(authentication.getEnUu());

    invoiceReplyDao.save(invoiceReply);

    notificationService.replyFromWarehouse(invoiceReply);
    return true;
  }

  @Override
  public boolean replyProdioMainfestFromWarehouse(ProdioMainfest mainfest) {
    Validate.notNull(mainfest, "回传信息不能为空");
    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    if (authentication == null) {
      throw new IllegalStateException("App Key无效");
    }
    mainfest.setSenderUU(authentication.getEnUu());

    prodioMainfestDao.save(mainfest);

    notificationService.replyFromWarehouse(mainfest);
    return true;
  }

  @Override
  public boolean replyProductMainfestFromWarehouse(ProductMainfest mainfest) {
    Validate.notNull(mainfest, "回传信息不能为空");
    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    if (authentication == null) {
      throw new IllegalStateException("App Key无效");
    }
    mainfest.setSenderUU(authentication.getEnUu());

    productMainfestDao.save(mainfest);

    notificationService.replyFromWarehouse(mainfest);
    return true;
  }

  @Override
  public boolean replyFeeFromWarehouse(Fee fee) {
    Validate.notNull(fee, "回传信息不能为空");
    AppAuthentication authentication = AuthenticationUtil.getAuthentication();
    if (authentication == null) {
      throw new IllegalStateException("App Key无效");
    }
    fee.setSenderUU(authentication.getEnUu());

    feeDao.save(fee);

    notificationService.replyFromWarehouse(fee);
    return true;
  }

  /**
   * CheckAppKey valid.
   * @param authentication
   */
  private void checkAppKey(AppAuthentication authentication){
    if (authentication == null || IdentificationType.ENTERPRISE_USER != authentication.getType()) {
      throw new IllegalStateException("App Key无效");
    }
  }
}
