package com.uas.platform.warehouse.backend.util;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.Map;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.Validate;
import org.apache.log4j.Logger;

/**
 * Api 接口参数签名工具类，负责进行请求参数的签名信息的生成.
 *
 * @author huxz
 */
public class SignUtil {

  private static final Logger logger = Logger.getLogger(SignUtil.class);

  /**
   * 根据Appkey和SecretKey生成签名字符串.
   *
   * @param appKey 申请接口时生成的key，标识用户
   * @param secretKey 根据AppKey生成的私钥
   * @param params 请求参数，包含时间戳和随机数
   * @return 签名字符串
   */
  public static String createSign(String appKey, String secretKey, Map<String, String> params) {
    Validate.notBlank(appKey, "App key不能为空");
    Validate.notBlank(secretKey, "Secret Key 不能为空");
    Validate.notNull(params, "请求参数不能为空");

    // 对参数名进行字典排序
    String[] keyArray = params.keySet().toArray(new String[0]);
    Arrays.sort(keyArray);

    if (logger.isDebugEnabled()) {
      logger.info("Sorted key: " + Arrays.toString(keyArray));
    }

    // 拼接有序的参数名-值串
    StringBuilder stringBuilder = new StringBuilder();
    stringBuilder.append(appKey);
    for (String key : keyArray) {
      stringBuilder.append(key).append(params.get(key));
    }
    stringBuilder.append(secretKey);
    String codes = stringBuilder.toString();

    return DigestUtils.sha1Hex(codes).toUpperCase();
  }

  /**
   * 根据Appkey和SecretKey生成MD5签名字符串.
   *
   * @param appKey 申请接口时生成的key，标识用户
   * @param secretKey 根据AppKey生成的私钥
   * @param timestamp 时间戳
   * @return 签名字符串
   */
  public static String createMD5Sign(String appKey, String secretKey, String timestamp) {
    Validate.notBlank(appKey, "App key不能为空");
    Validate.notBlank(secretKey, "Secret Key 不能为空");
    Validate.notNull(timestamp, "时间戳不能为空");

     return getMD5(appKey + timestamp + secretKey );
  }

  //!!注意：不足32位要补0
  private static String getMD5(String str) {
    try {
      MessageDigest md = MessageDigest.getInstance("MD5");
      md.update(str.getBytes());
      return new BigInteger(1, md.digest()).toString(16);
    } catch (Exception e) {
      throw new RuntimeException("MD5加密出现错误");
    }
  }
}
