package com.uas.platform.warehouse.tools.client;

import com.uas.platform.warehouse.core.common.ResultBean;
import com.uas.platform.warehouse.core.util.JacksonUtils;
import com.uas.platform.warehouse.tools.exception.RequestFailException;
import com.uas.platform.warehouse.tools.util.HttpClientUtil;
import com.uas.platform.warehouse.tools.util.SignUtil;
import com.uas.platform.warehouse.tools.util.UrlUtils;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Map;

/**
 * Abstract class is designed to hold configurations for SDK client.
 *
 * @author huxz
 */
public abstract class AbstractClient implements Client {

  protected String host;

  protected String appKey;

  protected String secretKey;

  /**
   * Constructs an instance.
   */
  public AbstractClient() {
  }

  /**
   * Constructs an instance to set host of warehouse backend app.
   *
   * @param host host string of warehouse backend app, non-empty
   */
  public AbstractClient(String host) {
    this.host = host;
  }

  /**
   * Constructs an instance to set host of warehouse backend app and authentication key.
   *
   * @param host host string of warehouse backend app, non-empty
   * @param appKey key represents app visitor
   * @param secretKey secret key using to generate sign
   */
  public AbstractClient(String host, String appKey, String secretKey) {
    this.host = host;
    this.appKey = appKey;
    this.secretKey = secretKey;
  }

  /**
   * update status
   */
  public boolean updateStatus(String data) throws Exception{
    List<NameValuePair> nvpList = getSignKey();
    URI uri= UrlUtils.buildUrl(host, "api/update-status", nvpList);
    ResultBean resultBean = HttpClientUtil.postMethod(uri,data);

    if (resultBean == null) {
      throw new ClientProtocolException("Response contains no content");
    } else if (!resultBean.getSuccess()) {
      throw new RequestFailException(resultBean.getMessage());
    }

    Map<String,Object> res = (Map<String,Object>)resultBean.getData();
    return Boolean.parseBoolean(res.get("success").toString());
  }

  /**
   * 将单据数据上传到B2B平台
   * @param data ，格式为[{}]
   * @return string
   * @throws Exception
   */
  @SuppressWarnings("unchecked")
  public boolean sendToB2b(String data)throws Exception{
    URI uri = getUri();
    ResultBean resultBean = HttpClientUtil.postMethod(uri,data);

    if (resultBean == null) {
      throw new ClientProtocolException("Response contains no content");
    } else if (!resultBean.getSuccess()) {
      throw new RequestFailException(resultBean.getMessage());
    }

    Map<String,Object> res = (Map<String,Object>)resultBean.getData();
    return Boolean.parseBoolean(res.get("success").toString());
  }

  public <T> boolean sendToB2b(List<T> datas)throws Exception{
    String data = JacksonUtils.toJson(datas);
    return sendToB2b(data);
  }

  /**
   * 获取上传到平台的URI
   * @return uri
   * @throws URISyntaxException uri解析错误
   */
  public URI getUri() throws URISyntaxException {
    List<NameValuePair> nvpList = getSignKey();
    return UrlUtils.buildUrl(host, "api/post", nvpList);
  }

  /**
   * 获取签名信息
   * @return 签名信息
   */
  public List<NameValuePair> getSignKey(){
    return SignUtil.generateParameterList(appKey, secretKey, null);
  }

  @Override
  public void setHost(String host) {
    this.host = host;
  }

  @Override
  public void setAppKey(String appKey) {
    this.appKey = appKey;
  }

  @Override
  public void setSecretKey(String secretKey) {
    this.secretKey = secretKey;
  }
}
